[{
  Name: 'Nest Thermostat Controller',
  Author: 'Andrew Seyes',
  Version: '1.0',
  
  // Version 1.0 - Initial Release

  IconImage: 'pages/control/widgets/nest/thermostat/images/nestThermostatOn.png',
  Initialized: false,

  RenderView: function (cuid, module) {
    var container      = $(cuid);
    var widget         = container.find('[data-ui-field=widget]');
    var _this          = this;
    
    if (!this.Initialized)
    {
      this.Initialized   = true;
      this.refresh       = 'INITIAL';                              // Indicates to Temp Up/Down functions if the widget has been refreshed.
      this.programDomain = 'HomeAutomation.HomeGenie.Automation';  // Domain and...
      this.programName   = 'Nest Thermostat';					   //  ... Name of the Nest Thermostat program.

      // settings button
      widget.find('[data-ui-field=settings]').on('click', function () {
        HG.WebApp.ProgramEdit._CurrentProgram.Domain  = 'HomeAutomation.HomeGenie.Automation';
        HG.WebApp.ProgramEdit._CurrentProgram.Address = programModule.Address
        HG.WebApp.ProgramsList.UpdateOptionsPopup();
      });
      // store a reference to ui fields
      this.autoText         = widget.find('[data-ui-field=autoText]');
      this.awayText         = widget.find('[data-ui-field=awayText]');
      this.battery          = widget.find('[data-ui-field=battery]');
      this.batteryImage     = widget.find('[data-ui-field=batteryImage]');
      this.currentHumidity  = widget.find('[data-ui-field=currentHumidity]');
	  this.currentTemp      = widget.find('[data-ui-field=currentTemperature]');
      this.endAwayButton    = widget.find('[data-ui-field=endAwayButton]');
      this.nestBanner       = widget.find('[data-ui-field=nestBanner]');
      this.heatSetToText    = widget.find('[data-ui-field=heatSetToText]');
      this.humidityImage    = widget.find('[data-ui-field=humidityImage]');
      this.iconImage        = widget.find('[data-ui-field=icon]');
      this.leafImage        = widget.find('[data-ui-field=leaf]');
      this.localIP          = widget.find('[data-ui-field=localIP]');
      this.location         = widget.find('[data-ui-field=location]');
      this.nestContact      = widget.find('[data-ui-field=nestContact]');
      this.nestStatus       = widget.find('[data-ui-field=nestStatus]');
      this.nestStatusImage  = widget.find('[data-ui-field=nestStatusImage]');
      this.refreshButton    = widget.find('[data-ui-field=refreshButton]');
      this.serialNumber     = widget.find('[data-ui-field=serialNumber]');
      this.setTempButton    = widget.find('[data-ui-field=setTempButton]');
      this.setAwayButton    = widget.find('[data-ui-field=setAwayButton]');
      this.targetTemp       = widget.find('[data-ui-field=targetTemperature]');
      this.tempArrowDown    = widget.find('[data-ui-field=tempArrowDown]');
      this.tempArrowUp      = widget.find('[data-ui-field=tempArrowUp]');
      this.tempDown         = widget.find('[data-ui-field=tempDown]');
      this.tempUp           = widget.find('[data-ui-field=tempUp]');
      this.timeToTargetText = widget.find('[data-ui-field=timeToTargetText]');
      this.wanIP            = widget.find('[data-ui-field=wanIP]');
      this.widgetUpdateTime = widget.find('[data-ui-field=widgetUpdateTime]');

      // Handle ui elements events
      // Set Away button
      this.setAwayButton.bind('click', function(){
        HG.Control.Modules.ServiceCall("SetAway", module.Domain, module.Address, "now", function (data) { });
        _this.nestStatus.html("Away Set");
        _this.nestStatusImage.hide();
      });
      // End Away button
      this.endAwayButton.bind('click', function(){
        HG.Control.Modules.ServiceCall("EndAway", module.Domain, module.Address, "now", function (data) { });
        _this.nestStatus.html("Away Ended...");
        _this.nestStatusImage.hide();
      });
      // Refresh button
      this.refreshButton.bind('click', function(){
        HG.Control.Modules.ServiceCall("Refresh", module.Domain, module.Address, "now", function (data) { });
        _this.nestStatus.html("Refresh in progress...");
        _this.nestStatusImage.hide();
      });
      // Set Temperature button
      this.setTempButton.bind('click', function(){
        HG.Control.Modules.ServiceCall("SetTemperature", module.Domain, module.Address, newtargettemp, function (data) { });
        _this.nestStatus.html("Temperature set to " + newtargettemp);
        _this.nestStatusImage.hide();
      });
      // Temperature Up Arrow
      this.tempUp.bind('click', function(){
        _this.ChangeTargetTemperature(module,'UP');
      });
      // Temperature Down Arrow
      this.tempDown.bind('click', function(){
        _this.ChangeTargetTemperature(module,'DOWN');        
      });      
    }

    // Get Program Module parmeters
    var programModule  = _this.GetProgramModuleByName(this.programDomain,this.programName);
    var parmCheck	   = HG.WebApp.Utility.GetModulePropertyByName(programModule,'Parameter.Check').Value;
    var parmCheckMsg   = HG.WebApp.Utility.GetModulePropertyByName(programModule,'Parameter.Check.Message').Value;
        
    // Warn if program configuration not complete
    if (parmCheck == 'Error') {
      this.nestBanner.attr('style', 'background:rgba(255,255,0,1);padding:6px;color:black;font-size:14pt;');
      this.nestBanner.html(parmCheckMsg);
    }

    if (parmCheck == 'OK') {
      // Get Module parmeters set by Nest Interface getDeviceInfo command
      var autoaway        = HG.WebApp.Utility.GetModulePropertyByName(module,'Auto.Away').Value;
      var battery         = HG.WebApp.Utility.GetModulePropertyByName(module,'Battery').Value;
      var currenthumidity = HG.WebApp.Utility.GetModulePropertyByName(module,'Current.Humidity').Value;
      var currenttemp     = HG.WebApp.Utility.GetModulePropertyByName(module,'Current.Temperature').Value;
      var heating         = HG.WebApp.Utility.GetModulePropertyByName(module,'Heating').Value;
      var leaf            = HG.WebApp.Utility.GetModulePropertyByName(module,'Leaf').Value;
      var localIP         = HG.WebApp.Utility.GetModulePropertyByName(module,'Local.IP').Value;
      var location		  = HG.WebApp.Utility.GetModulePropertyByName(module,'Location').Value;
      var manualaway      = HG.WebApp.Utility.GetModulePropertyByName(module,'Manual.Away').Value;
      var online          = HG.WebApp.Utility.GetModulePropertyByName(module,'Online').Value;
      var interfaceStatus = HG.WebApp.Utility.GetModulePropertyByName(module,'Interface.Status').Value;
      var serialnumber	  = HG.WebApp.Utility.GetModulePropertyByName(module,'Serial.Number').Value;
      var tempscale       = HG.WebApp.Utility.GetModulePropertyByName(module,'Temperature.Scale').Value;
      var timetotarget    = HG.WebApp.Utility.GetModulePropertyByName(module,'Time.To.Target').Value;
      var targettemp      = HG.WebApp.Utility.GetModulePropertyByName(module,'Target.Temperature').Value;
      var wanIP           = HG.WebApp.Utility.GetModulePropertyByName(module,'Wan.IP').Value;

      var lastconnectTime = new Date(HG.WebApp.Utility.GetModulePropertyByName(module,'Last.Connection').Value.replace(' ', 'T'));
      var updateTime      = new Date(HG.WebApp.Utility.GetModulePropertyByName(module,'Current.Temperature').UpdateTime.replace(' ', 'T'));
    
      // Set New Target Temperature (used by tempUp/tempDown functions) to targettemp
      var newtargettemp = targettemp;
      this.refresh = 'YES';  // Indicate to Temp Up/Down functions that a refresh has occurred

      // Set Nest status, temperature scale (degress C = '&#8451', degress F = '&#8457') and heat status
      var nestStatus = (online    == '1') ? 'Nest is Online' : 'Nest is Offline' 
      var nestScale  = (tempscale == 'C') ? '&#8451;' : '&#8457;'
      var heatStatus = (heating   == '1') ? 'On' : 'Off'
 
      // Default widget setup
      this.setTempButton.hide();
      this.timeToTargetText.hide();
      this.autoText.hide();
      this.awayText.hide();
      this.endAwayButton.hide();
      this.targetTemp.show();
      this.nestStatusImage.show();
      this.setAwayButton.show();
      this.tempArrowDown.show();
      this.tempArrowUp.show();
      this.humidityImage.show();
      this.batteryImage.show();

      // Display Nest Leaf image if required
      (leaf == 1) ? this.leafImage.show() : this.leafImage.hide();

      // If heating ON, set image with orange centre, turn heating banner orange and hide the "Heat Set To" text and calculate Time to Target if available
      if (heating == '1')  { 
        this.iconImage.attr('src', 'pages/control/widgets/nest/thermostat/images/nestThermostatOn.png');
        this.nestBanner.attr('style', 'background:rgba(255,100,0,1);padding:6px;color:white;font-size:14pt;');
        this.heatSetToText.hide();
        // Calculate Time to Target
        if (timetotarget > 0) {
          this.timeToTargetText.show();
          // Note: timetotarget is in seconds, so * 1000 to convert to milliseconds
          // Note: toUTCString format : Day, dd mmm yyyy hh:mm:ss GMT
          //                            Thu, 01 Jan 1970 01:39:29 GMT
          //                            01234567890123456789012345678
          var heatTarget     = new Date( (timetotarget * 1000) - Date.now() ).toUTCString();
          var UTChours       = parseInt(heatTarget.substring(17,19))
          var UTCmins        = parseInt(heatTarget.substring(20,22)); 
          var heatTargetText = "";
          switch (UTChours) {
            case 0:
              heatTargetText = (UTCmins > 30) ? 'IN 1 HR' : 'IN 1/2 HR' ;
              break;
            case 1:
              heatTargetText = 'IN 1+ HR';
              break;
            default:
              heatTargetText = 'IN 2+ HR';
              break;
          }
        }
      }
      // If heating OFF, set image with black centre, turn heating banner black and show the "Heat Set To" text
      else { 
        this.iconImage.attr('src', 'pages/control/widgets/nest/thermostat/images/nestThermostatOff.png');
        this.nestBanner.attr('style', 'background:rgba(0,0,0,1);padding:6px;color:white;font-size:14pt;');
        this.heatSetToText.show();
      }

      // Display Auto Away / Manual Away if required 
      if (autoaway == "1" || manualaway == "1" ) {
        this.autoText.hide();
        this.heatSetToText.hide();
        this.leafImage.hide();
        this.setAwayButton.hide();
        this.targetTemp.hide();
        this.tempArrowDown.hide();
        this.tempArrowUp.hide();
        if (autoaway == "1") this.autoText.show(); // Auto Away takes precedence if both Auto Away and Manual Away are set
        this.awayText.show();
        this.endAwayButton.show();
      }

      // Format the "widget update" time and Nest "last connect" time
      var formattedUpdateTime      = HG.WebApp.Utility.FormatDate(updateTime)                   + ' ' + HG.WebApp.Utility.FormatDateTime(updateTime);
      var formattedLastConnectTime = HG.WebApp.Utility.FormatDate(lastconnectTime).substring(4) + ' ' + HG.WebApp.Utility.FormatDateTime(lastconnectTime);
      
      // Update widget fields
      this.autoText.html         ('Auto');
      this.awayText.html         ('Away');
      this.battery.html          (battery          + 'v');
      this.currentTemp.html      (currenttemp      + nestScale);
      this.currentHumidity.html  (currenthumidity  + '%');
      this.nestBanner.html       ('Heating '       + heatStatus);
      this.heatSetToText.html    ('Heat Set To');
      this.localIP.html          (localIP);
      this.location.html         ('Nest - '        + location);
      this.nestStatus.html       (nestStatus);
      this.nestContact.html      ('Last contact: ' + formattedLastConnectTime);
	  this.serialNumber.html     ('Nest Serial# '  + serialnumber);
      this.targetTemp.html       (targettemp       + nestScale);
      this.timeToTargetText.html (heatTargetText);
      this.widgetUpdateTime.html ('Last updated: ' + formattedUpdateTime);
      
      // Warn if call to Nest Interface failed
      if (interfaceStatus == 'ERROR') {
        this.nestBanner.attr('style', 'background:rgba(255,000,0,1);padding:6px;color:white;font-size:14pt;');
        this.nestBanner.html('PHP script error, check Homegenie log');
      }
    
      // Warn if Nest is offline
      if (online != '1') {
        this.nestBanner.attr('style', 'background:rgba(255,255,0,1);padding:6px;color:black;font-size:14pt;');
        this.nestBanner.html('Warning - Nest is offline');
        this.nestStatusImage.attr('src', 'pages/control/widgets/nest/thermostat/images/red-cross.png');
      }

    }

   },

  // Get program module given domain and program name
  GetProgramModuleByName: function(domain,programName) {
    var module = null;
    for (var m = 0; m < HG.WebApp.Data.Modules.length; m++) {
        if (HG.WebApp.Data.Modules[m].Domain     == domain      && 
            HG.WebApp.Data.Modules[m].Name       == programName && 
            HG.WebApp.Data.Modules[m].DeviceType == 'Program') {
              module = HG.WebApp.Data.Modules[m];
            break;
        }
    }
    return module;
  },

  // Change the target temperature - invoked by the TempUp/Down functions
  ChangeTargetTemperature: function(module,request) {
    var heating       = HG.WebApp.Utility.GetModulePropertyByName(module,'Heating').Value;
    var leaf          = HG.WebApp.Utility.GetModulePropertyByName(module,'Leaf').Value;
    var targettemp    = HG.WebApp.Utility.GetModulePropertyByName(module,'Target.Temperature').Value;
    var tempscale     = HG.WebApp.Utility.GetModulePropertyByName(module,'Temperature.Scale').Value;
    
    var nestScale     = (tempscale == 'C') ? '&#8451;' : '&#8457;'

    var programModule = this.GetProgramModuleByName(this.programDomain,this.programName);
    var nestMaxTemp   = parseInt(HG.WebApp.Utility.GetModulePropertyByName(programModule,'ConfigureOptions.nestMaxTemperature').Value);
    var nestMinTemp   = parseInt(HG.WebApp.Utility.GetModulePropertyByName(programModule,'ConfigureOptions.nestMinTemperature').Value);
    
    if (this.refresh == 'YES') {
        newtargettemp = targettemp;  // Widget has been refreshed, so reset newtargetemp value to targettemp
        this.refresh = 'NO';
    }
    // Change new target temperature and check to see if we have hit the maximum/minimum value
    switch (request) {
      case 'UP':
        newtargettemp++;
        if (newtargettemp > nestMaxTemp) newtargettemp = nestMaxTemp; 
        break;
      case 'DOWN':
        newtargettemp--;
        if (newtargettemp < nestMinTemp) newtargettemp = nestMinTemp;
        break;
    }
    
    // Update Widget
    this.targetTemp.html (newtargettemp + nestScale);
    this.heatSetToText.hide();
    this.leafImage.hide();
    this.setAwayButton.hide();
    this.setTempButton.show();
    this.timeToTargetText.hide();
    if (newtargettemp == targettemp) {
      this.heatSetToText.show();
      (leaf == 1) ? this.leafImage.show() : this.leafImage.hide();
      this.setAwayButton.show();
      this.setTempButton.hide();
      if (heating   == '1') {
        this.heatSetToText.hide();
        this.timeToTargetText.show();
      } 
    } 
  },

}]